include misc.e

global function depth(object tstobj)
    -- checks depth (sequences within sequences) of
    -- the object being tested.  
    --     0 -> atom
    --    >0 -> depth of sequence (1 -> no subsequences)
    atom tstresult
    atom currdepth

    if atom(tstobj) then
        return 0
    else
        currdepth = 0
        for i = 1 to length(tstobj) do
            tstresult = depth(tstobj[i])
            if tstresult > currdepth then
                currdepth = tstresult
            end if
        end for
        return currdepth + 1
    end if

end function    -- end of depth

global function chkstr(object tstobj)
    -- check string
    -- returns true (1) or false (0) to indicate
    -- whether the object is a string of ascii
    -- characters.  If values outside of the normal
    -- ascii range are detected or subsequences 
    -- are detected then it is not a string.
    atom tstresult

    tstresult = depth(tstobj)
    if tstresult = 0 or tstresult > 1 then
        return 0
    else
        if length(tstobj) > 0 then
            tstresult = 1
            for i = 1 to length(tstobj) do
                if tstobj[i] < ' ' or tstobj[i] > '~' then
                    tstresult = 0
                end if
            end for
            return tstresult
        else
            return 0
        end if
    end if

end function   -- end of chkstr

global function handleEscCodes(object tstobj)
    -- function used by sprints to check for quoted
    -- or escaped sequences
    -- returns a string with escaped characters preserved
    sequence printstr
    printstr = {}
    for i = 1 to length(tstobj) do
        if tstobj[i] = '\\' or tstobj[i] = '\"' or tstobj[i] = '\'' then
            printstr &= '\\'
        end if
        printstr &= tstobj[i]
    end for
    return printstr

end function   -- end of handleEscCodes

global function sprints(object tstobj)
    -- print string
    -- tries to print a sequence in a useful way,
    -- one line per subsequence, allowing for
    -- escaped characters.
    atom tstresult
    sequence printstr
    printstr = {}

    tstresult = depth(tstobj)
    if tstresult = 0 then
        if floor(tstobj) = tstobj then
            printstr = sprintf("%d", tstobj)
        else
            printstr = sprintf("%g", tstobj)
        end if
    elsif tstresult = 1 then
        -- check for string
        tstresult = chkstr(tstobj)
        if tstresult = 1 then
            printstr = handleEscCodes(tstobj)
        else
            -- only one level deep but not string, 
            -- use standard euphoria "sprint" command
            printstr = sprint(tstobj)
        end if
    else
        -- handle a sequence
        printstr &= '{'
        for i = 1 to length(tstobj) do
            tstresult = chkstr(tstobj[i])
            if tstresult = 1 then
                printstr &= sprintf("\"%s\"",{handleEscCodes(tstobj[i])})
            else
                printstr &= sprints(tstobj[i])
            end if
            if i < length(tstobj) then
                printstr &= ','
            end if
        end for
        printstr &= '}'
    end if
    return printstr

end function     -- end sprints

-------------------------------------------------------
-- KeyValue functions
--  kvloc      utility used by other kv routines to do binary
--             search in order to find location for key.
--             returns {location, keyExistSw to say whether
--             it is in the file or not already}
--  kv	       return value from sequence for specified key
--  kvndx      return ndx into sequence for specified key
--  kvset      return sequence with inserted/updated 
--	       value for specified key
--  kvapp      return sequence with value appended to 
--	       value of specified key
--  kvpre      return sequence with value prepended to 
--	       value of specified key
--  kvdel      remove key/value pair from kvseq
--  kvdmp      print on stdout the key/value seqs, each on a new line

-- common variables used for kv processing
    atom seqMax
    atom seqMin
    atom seqPtr
    atom donesw 
    atom keyExistSw
    object result

global function kvloc(sequence kvseq, object kvkey)
-- set seqPtr where key is or should be.  
-- set keyExistSw to true/false for found/not found.
-- returns seqPtr and keyExistSw
    seqMax = length(kvseq)
    seqMin = 1
    if length(kvseq) = 0 then
        donesw = 1
        seqPtr = 1
        keyExistSw = 0
    else
        donesw = 0
    end if
    while donesw = 0 do
      seqPtr = floor((seqMax - seqMin) / 2)
      if seqPtr <= seqMin then
          seqPtr = seqMin
      end if
      result = compare(kvseq[seqPtr][1], kvkey)
      if result = 0 then
          donesw = 1
          keyExistSw = 1
      else 
          if seqMax > seqMin then
            if result = -1 then  -- kvseq[seqPtr][1] < kvkey
              if seqMax > seqPtr then
                seqMin = seqPtr + 1
              else
                -- seqPtr = seqMax and it isn't high enough yet
                donesw = 1
                seqPtr += 1
                keyExistSw = 0
              end if
            else   -- result = 1, kvseq[seqPtr][1] > kvkey
              if seqPtr > seqMin then
                seqMax = seqPtr - 1
              else
                donesw = 1
                keyExistSw = 0
              end if
            end if
          else  -- seqMax = seqMin, done checking
            donesw = 1
            keyExistSw = 0
            if result = -1 then  -- kvseq[seqPtr][1] < kvkey
                -- new entry would go after the current one
                seqPtr += 1
            end if
          end if
      end if
    end while
    return {seqPtr, keyExistSw}

end function -- end kvloc procedure

global function kv(sequence kvseq, object kvkey)
    -- returns the value associated with the specified key
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)
    if keyExistSw then
        result = kvseq[seqPtr][2]
    else
        result = {}
    end if
    return result

end function    -- end of kv

global function kvndx(sequence kvseq, object kvkey)
    -- returns the index into the sequence where the 
    -- key/value entry is located or else -1 if it
    -- is not found
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)
    if keyExistSw then
        result = seqPtr
    else
        result = -1
    end if
    return result

end function    -- end of kvndx

global function kvset (sequence kvseq, object kvkey, object kvval)
    -- returns an updated squence where the passed in sequence
    -- has had the specified key/value pair added
    sequence tmpseq
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)

    if keyExistSw then
        --printf(1,"Updating existing entry, ptr = %d\n", {result})
        kvseq[seqPtr][2] = kvval
    else
        if seqPtr > length(kvseq) then
            kvseq = append(kvseq,{kvkey, kvval})
        else
            tmpseq = append(kvseq[1..seqPtr-1],{kvkey, kvval})
            tmpseq &= kvseq[seqPtr..length(kvseq)]
            --printf(1,"Inserting entry, ptr = %d\n", {seqPtr})
            kvseq = tmpseq
        end if
    end if
    return kvseq

end function    -- end of kvset

global function kvapp (sequence kvseq, object kvkey, object kvval)
    -- returns an updated sequence where the value of the entry 
    -- specified by the key is updated to have the specified 
    -- value appended to it.
    sequence tmpseq
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)
    if keyExistSw then
        if atom(kvseq[seqPtr][2]) or chkstr(kvseq[seqPtr][2]) then
            kvseq[seqPtr][2] = append({kvseq[seqPtr][2]}, kvval)
            tmpseq = kvseq
        else
            kvseq[seqPtr][2] = append(kvseq[seqPtr][2], kvval)
            tmpseq = kvseq
        end if
    else
        tmpseq = kvset(kvseq, kvkey, kvval)
    end if
    return tmpseq

end function       -- end of kvapp

global function kvpre (sequence kvseq, object kvkey, object kvval)
    -- returns an updated sequence where the value of the entry 
    -- specified by the key is updated to have the specified 
    -- value prepended to it.
    sequence tmpseq
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)
    if keyExistSw then
        if atom(kvval) or chkstr(kvval) then
            kvseq[seqPtr][2] = append({kvval},kvseq[seqPtr][2])
            tmpseq = kvseq
        else
            kvseq[seqPtr][2] = append(kvval,kvseq[seqPtr][2])
            tmpseq = kvseq
        end if
    else
        tmpseq = kvset(kvseq, kvkey, kvval)
    end if
    return tmpseq
end function  -- kvpre

global function kvdel(sequence kvseq, sequence kvkey)
    -- returns a sequence with the key/value entry specified
    -- by the key has been deleted
    sequence ignoreresult

    ignoreresult = kvloc(kvseq, kvkey)
    if keyExistSw then
        kvseq = kvseq[1..seqPtr-1] & kvseq[seqPtr..length(kvseq)]
    end if
    return kvseq

end function  -- kvdel

global procedure kvdmp(sequence kvseq)
    -- displays the contents of the key/value sequence
    -- as a series of numbered lines
    for i = 1 to length(kvseq) do
        printf(1,"%d:%s\n",{i,sprints(kvseq[i])})
    end for

end procedure  -- kvdmp

global function skvdmp(sequence kvseq)
    -- returns a sequence containing the contents of
    -- a sequence, with imbedded CR/LF between each
    -- entry in the key/value sequence
    sequence tmpStr
    tmpStr = {}

    for i = 1 to length(kvseq) do
        tmpStr &= sprintf("%d:%s\r\n",{i,sprints(kvseq[i])})
    end for
    return tmpStr

end function  -- skvdmp

